@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0+
 /*
  * ATI Radeon Video card Framebuffer driver.
  *
@@ -5,8 +6,6 @@
  * Zhang Wei <wei.zhang@freescale.com>
  * Jason Jin <jason.jin@freescale.com>
  *
- * SPDX-License-Identifier:	GPL-2.0+
- *
  * Some codes of this file is partly ported from Linux kernel
  * ATI video framebuffer driver.
  *
@@ -20,6 +19,7 @@
 
 #include <command.h>
 #include <bios_emul.h>
+#include <env.h>
 #include <pci.h>
 #include <asm/processor.h>
 #include <linux/errno.h>
@@ -637,7 +637,8 @@ void *video_hw_init(void)
 
 	videomode = CONFIG_SYS_DEFAULT_VIDEO_MODE;
 	/* get video mode via environment */
-	if ((penv = getenv ("videomode")) != NULL) {
+	penv = env_get("videomode");
+	if (penv) {
 		/* deceide if it is a string */
 		if (penv[0] <= '9') {
 			videomode = (int) simple_strtoul (penv, NULL, 16);
